<?php
/**
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 *  @author    Jose Ivan Suazo <isuazo@pixel.hn>
 *  @copyright 2017-2018 Pixel S.A. de C.V.
 *  @license   https://opensource.org/licenses/Apache-2.0
 */

class PixelPaymentValidationModuleFrontController extends ModuleFrontController
{

    public function postProcess()
    {
        /**
         * Get current cart object from session
         */
        $cart = $this->context->cart;
        $authorized = false;

        /**
         * Verify if this module is enabled and if the cart has
         * a valid customer, delivery address and invoice address
         */
        $module_active = $this->module->active;
        $cart_id_customer = $cart->id_customer;
        $cart_delivery = $cart->id_address_delivery;
        $cart_id_address = $cart->id_address_invoice;

        if (!$module_active || $cart_id_customer == 0 || $cart_delivery == 0 || $cart_id_address == 0) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        /**
         * Verify if this payment module is authorized
         */
        foreach (Module::getPaymentModules() as $module) {
            if ($module['name'] == 'pixelpayment') {
                $authorized = true;
                break;
            }
        }

        if (!$authorized) {
            die($this->l('This payment method is not available.'));
        }

        /** @var CustomerCore $customer */
        $customer = new Customer($cart->id_customer);

        /**
         * Check if this is a vlaid customer account
         */
        if (!Validate::isLoadedObject($customer)) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        /**
         * Place the order
         */
        $this->module->validateOrder(
            (int) $this->context->cart->id,
            Configuration::get('PS_OS_PAYMENT'),
            (float) $this->context->cart->getOrderTotal(true, Cart::BOTH),
            $this->module->displayName,
            null,
            null,
            (int) $this->context->currency->id,
            false,
            $customer->secure_key
        );

        /**
         * Redirect the customer to the order confirmation page
         */
        Tools::redirect('index.php?controller=order-confirmation&id_cart='.
        (int)$cart->id.'&id_module='.
        (int)$this->module->id.'&id_order='.
        $this->module->currentOrder.'&key='.
        $customer->secure_key);
    }
    public function savecardtoken()
    {
        $this->ajax = true;
        // your code here
        die( "token" );
    }
}
