<?php
/**
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 *  @author    Jose Ivan Suazo <isuazo@pixel.hn>
 *  @copyright 2017-2018 Pixel S.A. de C.V.
 *  @license   https://opensource.org/licenses/Apache-2.0
 */

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;

if (!defined('_PS_VERSION_')) {
    exit;
}

class PixelPayment extends PaymentModule
{
    public $details;
    public $owner;
    public $address;
    public $extra_mail_vars;

    public function __construct()
    {
        $this->name = 'pixelpayment';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.6';
        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
        $this->author = 'Pixel Inc.';
        $this->controllers = array('validation');
        $this->is_eu_compatible = 1;
        $this->module_key = 'e9c4338866f986ee0ff3d82c082bf44e';

        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->l('PixelPay Payment Module');
        $this->description = $this->l('The easiest way to make payments online');
        $this->confirmUninstall = $this->l('Are you sure you want to delete your details?');

        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('No currency has been set for this module.');
        }

        if (!Configuration::get('PIXELPAY_MERCHANT_KEY')) {
            $this->warning = $this->l('No merchant key provided');
        }

        if (!Configuration::get('PIXELPAY_MERCHANT_HASH')) {
            $this->warning = $this->l('No merchant hash provided');
        }

        if (Configuration::get('PIXELPAY_ENDPOINT_ENABLED') && empty(Configuration::get('PIXELPAY_ENDPOINT_DOMAIN'))) {
            $this->warning = $this->l('No custom endpoint domain provided');
        }

        if (!Configuration::get('PS_SSL_ENABLED')) {
            $this->warning = $this->l('You must enable SSL on the store if you want to use this module');
        }
    }

    public function install()
    {
        // Run sql for creating DB tables
        Db::getInstance()->execute('
                CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'pixel_tokens` (
                `id_token` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY ,
                `id_customer` INT(11) UNSIGNED NOT NULL,
                `token` VARCHAR(255) NOT NULL,
                `type` VARCHAR(255) NOT NULL,
                `card_token` VARCHAR(255) NOT NULL,
                `reference` VARCHAR(255) NOT NULL
            ) ENGINE = ' . _MYSQL_ENGINE_ . ' CHARACTER SET utf8 COLLATE utf8_general_ci;');

        if (!parent::install() || !$this->registerHook('paymentOptions') || !$this->registerHook('paymentReturn')) {
            return false;
        }

        $configurate_key_merchant = Configuration::updateValue('PIXELPAY_MERCHANT_KEY', '');
        $configurate_hash_merchant = Configuration::updateValue('PIXELPAY_MERCHANT_HASH', '');

        if (!$configurate_key_merchant || !$configurate_hash_merchant) {
            return false;
        }

        return true;
    }

    public function uninstall()
    {

        // SQL
        $sql = 'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'pixel_tokens`';
        if (Db::getInstance()->execute($sql) == false) {
            return false;
        }


        $config = array(
            'PIXELPAY_MERCHANT_KEY',
            'PIXELPAY_MERCHANT_HASH',
            'PIXELPAY_ENDPOINT_ENABLED',
            'PIXELPAY_ENDPOINT_DOMAIN'
        );

        foreach ($config as $var) {
            Configuration::deleteByName($var);
        }

        return parent::uninstall();
    }

    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        //$paymen_type = Configuration::get('PIXELPAY_INLINE_ENABLED');
        $paymen_type = Configuration::get('PIXELPAY_INLINE_TOKENIZATION_ENABLED');

        if ($paymen_type != 0) {
            $payment_options = array(
                $this->getEmbeddedPaymentOptionSdk()
            );
        } else {
            $payment_options = array(
                $this->getEmbeddedPaymentOption()
            );
        }



        return $payment_options;
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    public function savecardtoken(){
        return "yes";
    }
    public function getEmbeddedPaymentOptionSdk()
    {
        $formAction = $this->context->link->getModuleLink($this->name, 'validation', array(), true);
        $templateInfo = 'module:pixelpayment/views/templates/front/payment_infos.tpl';
        $embeddedOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $embeddedOption->setCallToActionText($this->l('Pay with Credit or Debit Card'))
            ->setForm($this->generateFormSdk())
            ->setAction($formAction)
            ->setAdditionalInformation($this->context->smarty->fetch($templateInfo));

        return $embeddedOption;
    }

    public function getEmbeddedPaymentOption()
    {
        $templateInfo = 'module:pixelpayment/views/templates/front/payment_infos_checkout.tpl';
        $embeddedOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $embeddedOption->setCallToActionText($this->l('Pay with Credit or Debit Card'))
            ->setForm($this->generateForm())
            ->setAdditionalInformation($this->context->smarty->fetch($templateInfo));

        return $embeddedOption;
    }

    public function getTokens($actual_customer)
    {
        // $type = "customer_token";
         $sql = 'Select * from `' . _DB_PREFIX_ . 'pixel_tokens` where id_customer = ' . $actual_customer .' and type = "customer_token"';

     

        return Db::getInstance()->executeS($sql);
    }

    protected function generateFormSdk()
    {
        $key = Configuration::get('PIXELPAY_MERCHANT_KEY');
        $hash = md5(Configuration::get('PIXELPAY_MERCHANT_HASH'));
        $endpoint_enabled = Configuration::get('PIXELPAY_ENDPOINT_ENABLED');
        $endpoint_domain = Configuration::get('PIXELPAY_ENDPOINT_DOMAIN');
        $enable_tokenization = Configuration::get('PIXELPAY_INLINE_TOKENIZATION_ENABLED');

        $endpoint = "https://pixelpay.app";

        if ($endpoint_enabled) {
            $endpoint = 'https://' . $endpoint_domain;
        }

        // Si hay que tokenizar
        $actual_customer =  $this->context->cart->id_customer;
        $tokens_activos = $this->getTokens($actual_customer);

        if (empty($tokens_activos)) {
            $postRequest = array(
                'email' => $this->context->customer->email,
            );

            $endpoint_customer = $endpoint_domain . '/api/v2/tokenization/customer';

            $client =  new Client();

            $resp = $client->post($endpoint_customer, [
                'headers' => [
                    'Accept' => 'application/json',
                    'x-auth-key' => $key,
                    'x-auth-hash' => $hash,
                ],
                'body' => $postRequest,
            ]);

            $resp_decode = json_decode($resp->getBody());

            $customer_token = $resp_decode->data->token;
            // No tiene tokens, entonces hay que generar un nuevo customer token y asociarlo
            $query_insert = 'INSERT INTO ' . _DB_PREFIX_;
            $type = "customer_token";
            $query = 'pixel_tokens (id_customer, token, type) VALUES (' . $actual_customer . ', "' . $customer_token . '", "' . $type . '" )';
            $query_string = $query_insert . $query;

            if (Db::getInstance()->execute($query_string) == false) {
                return Db::getInstance()->getMsgError();
            }

            // asignamos los tokens activos del customer
            $tokens_activos = $this->getTokens($actual_customer);
        }

        // consultar tarjetas asociadas a un cliete mediante customer_token
        try {
            $cliente = new Client();
            $url = $endpoint_domain . '/api/v2/tokenization/customer/';

            $res = $cliente->get($url . $tokens_activos[0]['token'], [
                'headers' => [
                    'x-auth-key' => $key,
                    'x-auth-hash' => $hash,
                ],
            ]);

            $res_decode = json_decode($res->getBody());
            $tarjetas = $res_decode->data;
        } catch (ClientException $e) {
            $e;
        }

        $cart = $this->context->cart;
        $currency = $this->context->currency;
        $address = new Address((int) $cart->id_address_invoice);
        $country_iso = Country::getIsoById($address->id_country);
        $ajax_link = Context::getContext()->link->getModuleLink('pixelpayment', 'pixelpay', array());
        $customer = new Customer((int) $cart->id_customer);
        $state = new State((int) $address->id_state);
        $pixel_token = '';
        $sql = 'Select * from ' . _DB_PREFIX_ . 'pixel_tokens where id_customer = ' . $cart->id_customer . ' ';
        $data  = Db::getInstance()->executeS($sql);
        if (!empty($data)) {
            $pixel_token = $data[0]['token'];
        }

        if (empty($state)) {
            $state = "Sin Estado";
        }

        $totalOnlyProductsWithTax = ((float) $cart->getOrderTotal(true, Cart::ONLY_PRODUCTS));
        $totalOnlyProducts = ((float) $cart->getOrderTotal(false, Cart::ONLY_PRODUCTS));

        $amount = (float) $cart->getOrderTotal(true, Cart::BOTH);
        $tax = $totalOnlyProductsWithTax - $totalOnlyProducts;
        $shipping = (float) $cart->getOrderTotal(true, Cart::ONLY_SHIPPING, null, $cart->id_carrier);
        $action = $this->context->link->getModuleLink($this->name, 'validation', array('order' => $cart->id), true);
        $cancel = $this->context->link->getPageLink('order&step=4', true);

        $products = $cart->getProducts();
        $items = array();

        foreach ($products as $product) {
            $items[] = array(
                'code' => $product['reference'] ? $product['reference'] : $product['id_product'],
                'title' => $product['name'],
                'description' => $product['attributes'],
                'price' => $product['price'],
                'qty' => $product['quantity'],
                'total' => $product['total']
            );
        }

        $items = count($items) > 0 ? json_encode($items) : null;

        if (!Validate::isLoadedObject($customer) && Tools::getValue('step') != 1) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        $this->context->smarty->assign(array(
            'action' => $action,
            'cancel' => $cancel,
            'amount' => $amount,
            'tax' => $tax,
            'shipping' => $shipping,
            'address' => $address,
            'state' => $state->iso_code,
            'currency' => $currency,
            'date' => $cart->date_add,
            'order' => str_pad($cart->id, 8, 0, STR_PAD_LEFT),
            'customer' => $customer,
            'items' => $items,
            'key' => $key,
            'endpoint' => $endpoint,
            'hash' => $hash,
            'pixelpayPath' => Media::getMediaPath(_PS_MODULE_DIR_ . $this->name),
            'pixel_token' => $pixel_token,
            'country_iso' => $country_iso,
            'ajax_link' => $ajax_link,
            'cards' => $tarjetas,
            'tokenization' => $enable_tokenization,
        ));

        return $this->context->smarty->fetch('module:pixelpayment/views/templates/front/payment_sdk_form.tpl');
    }

    protected function generateForm()
    {
        $key = Configuration::get('PIXELPAY_MERCHANT_KEY');
        $hash = Configuration::get('PIXELPAY_MERCHANT_HASH');
        $endpoint_enabled = Configuration::get('PIXELPAY_ENDPOINT_ENABLED');
        $endpoint_domain = Configuration::get('PIXELPAY_ENDPOINT_DOMAIN');

        $endpoint = "pixelpay.app";

        if ($endpoint_enabled) {
            $endpoint = $endpoint_domain;
        }



        $cart = $this->context->cart;
        $currency = $this->context->currency;
        $address = new Address((int) $cart->id_address_invoice);
        $customer = new Customer((int) $cart->id_customer);
        $state = State::getNameById($address->id_state);

        if (empty($state)) {
            $state = "Sin Estado";
        }

        $totalOnlyProductsWithTax = ((float) $cart->getOrderTotal(true, Cart::ONLY_PRODUCTS));
        $totalOnlyProducts = ((float) $cart->getOrderTotal(false, Cart::ONLY_PRODUCTS));

        $amount = (float) $cart->getOrderTotal(true, Cart::BOTH);
        $tax = $totalOnlyProductsWithTax - $totalOnlyProducts;
        $shipping = (float) $cart->getOrderTotal(true, Cart::ONLY_SHIPPING, null, $cart->id_carrier);
        $action = $this->context->link->getModuleLink($this->name, 'validation', array('order' => $cart->id), true);
        $cancel = $this->context->link->getPageLink('order&step=4', true);

        $products = $cart->getProducts();
        $items = array();

        foreach ($products as $product) {
            $items[] = array(
                'code' => $product['reference'] ? $product['reference'] : $product['id_product'],
                'title' => $product['name'],
                'description' => $product['attributes'],
                'price' => $product['price'],
                'qty' => $product['quantity'],
                'total' => $product['total']
            );
        }

        $items = count($items) > 0 ? json_encode($items) : null;

        if (!Validate::isLoadedObject($customer) && Tools::getValue('step') != 1) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        $this->context->smarty->assign(array(
            'action' => $action,
            'cancel' => $cancel,
            'amount' => $amount,
            'tax' => $tax,
            'shipping' => $shipping,
            'address' => $address,
            'state' => $state,
            'currency' => $currency,
            'date' => $cart->date_add,
            'order' => str_pad($cart->id, 8, 0, STR_PAD_LEFT),
            'customer' => $customer,
            'items' => $items,
            'key' => $key,
            'endpoint' => $endpoint,
            'hash' => $hash,
            'pixelpayPath' => Media::getMediaPath(_PS_MODULE_DIR_ . $this->name),
        ));

        return $this->context->smarty->fetch('module:pixelpayment/views/templates/front/payment_form.tpl');
    }

    public function getContent()
    {
        $output = null;

        if (Tools::isSubmit('submit' . $this->name)) {
            $module_key = (string) Tools::getValue('PIXELPAY_MERCHANT_KEY');
            $module_hash = (string) Tools::getValue('PIXELPAY_MERCHANT_HASH');
            $endpoint_enabled = (bool) Tools::getValue('PIXELPAY_ENDPOINT_ENABLED');
            $endpoint_domain = (string) Tools::getValue('PIXELPAY_ENDPOINT_DOMAIN');
            $module_inline = (string) Tools::getValue('PIXELPAY_INLINE_ENABLED');
            $module_inline_tokenization = (string) Tools::getValue('PIXELPAY_INLINE_TOKENIZATION_ENABLED');
            $va_key = Validate::isGenericName($module_key);
            $va_hash = Validate::isGenericName($module_hash);

            if (!$module_key || !$module_hash || empty($module_key) || empty($module_hash) || !$va_key || !$va_hash) {
                $output .= $this->displayError($this->l('Invalid Configuration values'));
            } else {
                Configuration::updateValue('PIXELPAY_MERCHANT_KEY', $module_key);
                Configuration::updateValue('PIXELPAY_MERCHANT_HASH', $module_hash);
                Configuration::updateValue('PIXELPAY_ENDPOINT_ENABLED', $endpoint_enabled);
                Configuration::updateValue('PIXELPAY_ENDPOINT_DOMAIN', $endpoint_domain);
                Configuration::updateValue('PIXELPAY_INLINE_ENABLED', $module_inline);
                Configuration::updateValue('PIXELPAY_INLINE_TOKENIZATION_ENABLED', $module_inline_tokenization);

                $output .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }

        return $output . $this->displayForm();
    }

    public function displayForm()
    {
        $fields_form = array();

        // Get default language
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');

        // Init Fields form array
        $fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('PixelPay Settings'),
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => $this->l('PixelPay Key Account'),
                    'name' => 'PIXELPAY_MERCHANT_KEY',
                    'hint' => $this->l('Extract'),
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('PixelPay Secret Key Account'),
                    'name' => 'PIXELPAY_MERCHANT_HASH',
                    'hint' => $this->l('Extract'),
                    'size' => 42,
                    'required' => true
                ),
                array(
                    'type' => 'switch',
                    'label' => 'Enable PixelPay custom endpoint?',
                    'name' => 'PIXELPAY_ENDPOINT_ENABLED',
                    'is_bool' => true,
                    'required' => false,
                    'values' => array(
                        array(
                            'id' => 'pp_endpoint_on',
                            'value' => 1,
                            'label' => $this->trans('Enabled', [], 'Admin.Global')
                        ),
                        array(
                            'id' => 'pp_endpoint_off',
                            'value' => 0,
                            'label' => $this->trans('Disabled', [], 'Admin.Global')
                        )
                    )
                ),
                array(
                    'type' => 'text',
                    'label' => 'PixelPay custom endpoint domain',
                    'name' => 'PIXELPAY_ENDPOINT_DOMAIN',
                    'size' => 60,
                    'required' => false
                ),
                /* array(
                    'type' => 'switch',
                    'label' => 'Use Inline Payments',
                    'name' => 'PIXELPAY_INLINE_ENABLED',
                    'is_bool' => true,
                    'required' => false,
                    'values' => array(
                        array(
                            'id' => 'pp_inline_on',
                            'value' => 1,
                            'label' => $this->trans('Enabled', [], 'Admin.Global')
                        ),
                        array(
                            'id' => 'pp_inline_off',
                            'value' => 0,
                            'label' => $this->trans('Disabled', [], 'Admin.Global')
                        )
                    )
                ), */
                /*array(
                    'type' => 'switch',
                    'label' => 'Activate tokenization',
                    'name' => 'PIXELPAY_INLINE_TOKENIZATION_ENABLED',
                    'is_bool' => true,
                    'required' => false,
                    'values' => array(
                        array(
                            'id' => 'pp_inline_tokenization_on',
                            'value' => 1,
                            'label' => $this->trans('Enabled', [], 'Admin.Global')
                        ),
                        array(
                            'id' => 'pp_inline_tokenization_off',
                            'value' => 0,
                            'label' => $this->trans('Disabled', [], 'Admin.Global')
                        )
                    )
                ),*/
            ),
            'submit' => array(
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right'
            )
        );

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;

        // Language
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit' . $this->name;
        $helper->toolbar_btn = array(
            'save' =>
            array(
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&save' . $this->name .
                    '&token=' . Tools::getAdminTokenLite('AdminModules'),
            ),
            'back' => array(
                'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );

        // Load current value
        $enable_token = Configuration::get('PIXELPAY_INLINE_TOKENIZATION_ENABLED');
        $helper->fields_value['PIXELPAY_MERCHANT_KEY'] = Configuration::get('PIXELPAY_MERCHANT_KEY');
        $helper->fields_value['PIXELPAY_MERCHANT_HASH'] = Configuration::get('PIXELPAY_MERCHANT_HASH');
        $helper->fields_value['PIXELPAY_ENDPOINT_ENABLED'] = Configuration::get('PIXELPAY_ENDPOINT_ENABLED');
        $helper->fields_value['PIXELPAY_ENDPOINT_DOMAIN'] = Configuration::get('PIXELPAY_ENDPOINT_DOMAIN');
        $helper->fields_value['PIXELPAY_INLINE_ENABLED'] = Configuration::get('PIXELPAY_INLINE_ENABLED');
        $helper->fields_value['PIXELPAY_INLINE_TOKENIZATION_ENABLED'] = $enable_token;

        return $helper->generateForm($fields_form);
    }
}
